;; clojure-like namespaces
#|
usage:

(ns foo.bar)
(ns-require 'some.lib :as 'lib)
(ns-require 'some.lib.other :as 'other)

Or, more clojure like syntax

(ns foo.bar
  :doc "My foo.bar library"
  :require ((some.lib :as lib)
	    (other.lib :as other)))
|#
(require s7bi) ;; for (comment .. )

(define-expansion (comment . body) #<unspecified>)

(define (print . args)
  (for-each
   (lambda (x)
     (format *stderr* "~A " x))
   args)
  (format *stderr* "\n"))


;; returns the last argument
;; useful for in-drop debugging, printing what we return
(define (print-ret . args)
  (apply print args)
  (if (pair? args)
      (car (reverse args))
      ()))

;; hm that fails
;; (define-expansion (comment . body) (values))

(define map-indexed
  (let ((+documentation+ "(map-indexed f coll)
Applies f to the collection coll.
f should accept to arguments, i and the element")
        (+signature+ '(f coll)))
    (lambda (f coll)
      (let ((i -1))
        (map (lambda (el)
               (set! i (+ 1 i))
               (f i el))
             coll)))))

;; meta
(define (meta f)
  (if (and (procedure? f)
           ((funclet f) 'meta)
           (not (eq? meta ((funclet f) 'meta))))
      (((funclet f) 'meta) f)
      #f))

(provide 's7bi.prelude)
