#include <unordered_map>
#include <stdio.h>
#include "s7.h"

#include "./resources/main_scm.h"
#include "./resources/core_scm.h"

struct Resource {
  unsigned char* data;
  size_t size;
};

using Store = std::unordered_map<std::string, Resource>;

std::unordered_map<s7_scheme *, Store> stores;

void set(s7_scheme *sc, std::string name, Resource r) { stores[sc][name] = r; }

Resource get(s7_scheme *sc, std::string name) { return stores[sc][name]; }

s7_pointer load_with_resources(s7_scheme *sc, s7_pointer args) {
  printf("here load_with_resources\n");
  s7_pointer sc_file = s7_car(args);
  const char *file = s7_string(s7_car(args));
  // TODO setting s7_rootlet causes segmentation fault
  s7_pointer env = s7_nil(sc);
  args = s7_cdr(args);
  if (args != s7_nil(sc)) {
    env = s7_car(args);
  }

  Resource r = get(sc, file);
  if (!r.data) {
    printf("Failed to load %s as embedded resource, trying normal load\n", file);
    s7_pointer res = s7_load_with_environment(sc, file, env);
    // res might be null, should be careful to not return this
    // since this is an s7 binding
    if (!res) {
      return (s7_error(
          sc, s7_make_symbol(sc, "error"),
          s7_list(sc, 2, s7_make_string(sc, "load: can't open ~S"), sc_file)));
    }
    return res;
  }

  printf("Loading %s as embedded resource\n", file);
  s7_pointer res = s7_load_c_string_with_environment(sc, (char *)r.data, r.size,
                                                     env);
  if(!res) {
    printf("null res??\n");
    return s7_f(sc);
  }
  return res;
}

int main(int argc, char **argv)
{
  s7_scheme *sc = s7_init();
  s7_provide(sc, "s7bi");

  set(sc, "main.scm", {.data = main_scm, .size = main_scm_len});
  set(sc, "core.scm", {.data = core_scm, .size = core_scm_len});

  s7_define_function(sc, "load", load_with_resources, 1, 1, false,
                     "(load file (let (rootlet)))\n\
Tries to load \"file\" as an embedded resource first, and if it fails it fallbacks to s7's load function");

  // main.scm last sexp is (+ 1 2), so we should get 3 as a result
  s7_pointer res = s7_eval_c_string(sc, "(load \"main.scm\")");
  if(!s7_is_integer(res)) {
    printf("not an integer!\n");
    return 1;
  }

  return 0;
}
